/*
This is a part of the LiteStep Shell Source code.

Copyright (C) 1997-2001 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/ 
/*
  Copyright (C) 1998-1999 Johan Redestig
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/****************************************************************************
06/08/02 - Alessandro Limonta (allelimo)
  - PopupGradientSelEntry
  - PopupGradientSelFolder
05/22/02 - Erik Christiansson (Sci)
  - Shadow defaults
05/20/02 - Erik Christiansson (Sci)
  - Moved a lot of vars from PopupMaker to PopupMaker::Initialize to save memory
  - Added text shadows, Popup[Title|Entry|SelEntry|Folder|SelFolder][ShadowDepth|ShadowColor]
05/19/02 - Erik Christiansson (Sci)
  - Added additional font settings Popup[Entry|SelEntry|Folder|SelFolder|Title][Bold|Italic|Underline]
  - Added SelFolder font settings, defaults to SelEntry settings
05/15/02 - Erik Christiansson (Sci)
  - Fixed margins and spacing on !RemoteAmp
04/30/02 - Alessandro Limonta (allelimo)
  - Merged in the code by sol (stand-alone popup with folder contents)
12/19/00 - Bobby G. Vinyard (Message)
  - Added !PopupRecycleBin and !PopupRecentDocuments (this works correctly
    under W2K where as !DynamicFolder:$recent$ display everything
11/27/00 - Joachim Calvert (NeXTer)
  - Fixed the RevID so that the right version number is reported
11/15/00 - Kevin Schaffer (Maduin)
  - added PopupDefaultIcon, PopupIconSpacing, and PopupIconSize
  - added PopupTitleAlignment and PopupEntryAlignment
  - fixed a resource leak - MenuItem::~MenuItem() used DeleteObject to destroy an icon
11/12/00 - Bobby G. Vinyard (Message)
  - Removed 2000-10-18 additions to popup2 for right click open
  - Implemented right click context menus for the following items: !PopupFolder,
    !PopupDynamicFolder, !DynamicFolder, !PopupMyComputer, !PopupNetwork, !PopupControlPanel,
    !PopupPrinters (this code is by no means perfect, i.e. sub menus on the context menu
    do not function, i.e. the send to menu)
10/15/00 - Bobby G. Vinyard (Message)
  - Right clicking on a menu item that is a !PopupFolder or !PopupDynamicFolder, or one of
    of their subfolders. will cause the directory to be opened in a single pane explorer
    window
09/21/00 - Kevin Schaffer (Maduin)
  - Added .icon support
  - Added NoPopupTitleText
  - PopupBottomPix can now be a different height than PopupTitlePix
09/18/00 - Kevin Schaffer (Maduin)
  - Added support for some useful shell folders
    - !PopupMyComputer
	- !PopupNetwork
	- !PopupControlPanel
	- !PopupPrinters
08/31/00 - Joachim Calvert (NeXTer)
  - Altered the WM_POPUP message to pass both mouse coords in lParam
08/28/00 - Joachim Calvert (NeXTer)
  - Now handles scrolling of long menus, courtesy of Sylvain Rouquette (Syl)
  - New setting: PopupScrollSpeed <int>, default is 1
  - Errata: PopupTitleBold only works in conjunction with PopupFontFace, due
    to the method for retreiving the default font
08/27/00 - Joachim Calvert (NeXTer)
  - Should now handle gradients gracefully on newer builds, let me know if any
    problems occur
  - Added the settings PopupTitleBold <bool> and PopupEntryFontHeight <int>
  - Made !DynamicFolder menus display the caption of their parent
  - Fixed the vertical alignment of the text, should look better now
08/24/00 - Joachim Calvert (NeXTer)
  - Changed the min width default to 128px
08/17/00 - Joachim Calvert (NeXTer)
  - Hidden files and folders are no longer shown
  - Hides the extensions for .bat and .pif files
08/15/00 - Bobby G. Vinyard (Message)
  - Fixed a bug in !PopupDynamicFolder not showing
08/05/00 - Joachim Calvert (NeXTer)
  - Made the folder arrow static (doesn't resize)
  - Now takes the folder arrow into account when calculating the width of items
07/21/00 - Bobby G. Vinyard (Message)
  - Added !PopupDynamicFolder for compatiability with old Popup
  - Added folder merging capability to !DynamicFolder, !PopupDynamicFolder
    and !PopupFolder
****************************************************************************/
#include "stdafx.h"
#include "PopupMaker.h"
#include <winuser.h> 
// menu items
#include "MenuItem.h"
#include "FolderItem.h"
#include "PopupMenu.h"
#include "SeparatorItem.h"
#include "CommandItem.h"
#include "TimeItem.h"
#include "RemoteAmpItem.h"
#include "HeaderItem.h"
#include "DirectoryFolder.h"
#include "RunItem.h"
#include "TaskFolder.h"
#include "TaskItem.h"
#include "DesktopMenu.h"
#include "TaskMenu.h"
#include "ShellFolder.h"
#include "Re5ources.h"
#include "info.h"

// Painters
#include "BmpPainter.h" 
#include "GradientPainter.h"
#include "SolidPainter.h"

#include <io.h>
#include <shlwapi.h>
#include <vector>
#include <algorithm>

extern vector<PopupMenu*> g_PopupMenues;


PopupMaker::PopupMaker()
{
	m_pRoot = NULL;
	m_pBackground = NULL;
	m_pSeparator = NULL;
	m_pBottom = NULL;
}

PopupMaker::~PopupMaker()
{
	// unregister LS messages
	if (m_pRoot)
	{
        int msgs[] = { LM_POPUP, LM_HIDEPOPUP, LM_GETREVID, 0 };

		SendMessage(GetLitestepWnd(), LM_UNREGISTERMESSAGE,
            (WPARAM)m_pRoot->GetWindow(), (LPARAM) msgs);
	}

	DeleteObject(m_hfnTitle);
	DeleteObject(m_hfnEntry);
	DeleteObject(m_hfnFolder);
	DeleteObject(m_hfnSelEntry);
	DeleteObject(m_hfnSelFolder);

	delete m_pBackground;
	delete m_pTitle;
	delete m_pEntry;
	delete m_pSelEntry;
	delete m_pSeparator;
	delete m_pBottom;
	delete m_pFolder;
	delete m_pSelFolder;

	if (m_hbmArrow)
	{
		DeleteObject(m_hbmArrow);
		DeleteObject(m_hbmSelArrow);
	}

	// delete all PopupMenu object, this is important not just only for
	// not leaking memory but (which is important since one recycles
	// litestep from time to time. litestep will crash upon recycle otherwise
	while (!g_PopupMenues.empty())
		delete g_PopupMenues[0];

	// delete the painters that we have allocated.
	for (int i = 0; i < m_Painters.size(); i++)
		delete m_Painters[i];
}

void PopupMaker::Show(int index)
{
	m_RootMenus[index]->Show();
	SetFocus(m_RootMenus[index]->GetWindow());
}

void PopupMaker::Hide(int index)
{
	for (size_t i = 0; i < m_RootMenus.size(); i++)
	{
		m_RootMenus[i]->Hide(HIDE_PARENTS | HIDE_CHILDREN);
	}
}

void PopupMaker::ShowDesktopMenu()
{
	m_pDesktopMenu->Show();
	SetFocus(m_pDesktopMenu->GetWindow());
}

void PopupMaker::ShowTaskMenu()
{
	m_pTaskMenu->Show();
	SetFocus(m_pTaskMenu->GetWindow());
}

void PopupMaker::Initialize(HWND hParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
	char pszTitlePix[MAX_RC_STRING];
	char pszEntryPix[MAX_RC_STRING];
	char pszSelEntryPix[MAX_RC_STRING];
	char pszFolderPix[MAX_RC_STRING];
	char pszSelFolderPix[MAX_RC_STRING];
	char pszSeparatorPix[MAX_RC_STRING];
	char pszBottomPix[MAX_RC_STRING];
	char pszFontFace[MAX_RC_STRING];
	char pszEntryFontFace[MAX_RC_STRING];
	char pszFolderFontFace[MAX_RC_STRING];
	char pszSelEntryFontFace[MAX_RC_STRING];
	char pszSelFolderFontFace[MAX_RC_STRING];
	
	BOOL bGradientTitle;
	DWORD nGradientTitle;
	BOOL bGradientEntry;
	DWORD nGradientEntry;
	BOOL bGradientSelEntry;
	DWORD nGradientSelEntry;
	BOOL bGradientFolder;
	DWORD nGradientFolder;
	BOOL bGradientSelFolder;
	DWORD nGradientSelFolder;

	DWORD nTitleColor;
	DWORD nInActiveTitleColor;
	DWORD nEntryColor;
	DWORD nSelEntryColor;
	DWORD nFolderColor;
	DWORD nSelFolderColor;

	DWORD nTitleTextColor;
	DWORD nEntryTextColor;
	DWORD nSelEntryTextColor;
	DWORD nFolderTextColor;
	DWORD nSelFolderTextColor;

	DWORD nBevelLightColor;
	DWORD nBevelDarkColor;

	hInst = dllInst;
	int tmpInteger = 0;

	// we are managing the system errors our seleves. this will prevent
	// the system to show a error message box if we would try to l
	UINT nErrorMode = SetErrorMode(1);

	m_bTransparent = GetRCBool("PopupNotTransparent", FALSE); // draw transparent by default

	m_bHeader = GetRCBool("PopupNoTitles", FALSE);
	m_bHeaderText = GetRCBool("PopupNoTitleText", FALSE);

	//
    // Set the painters ...
    //
	GetRCString("PopupTitlePix", pszTitlePix, "", MAX_RC_STRING);
	GetRCString("PopupEntryPix", pszEntryPix, "", MAX_RC_STRING);
	GetRCString("PopupSelEntryPix", pszSelEntryPix, "", MAX_RC_STRING);
	GetRCString("PopupFolderPix", pszFolderPix, pszEntryPix, MAX_RC_STRING);
	
    if(stricmp(pszFolderPix, pszEntryPix) == 0)
    {
		DirectoryFolder::SetPaintFolderIcon(TRUE);
	}
	else
    {
		DirectoryFolder::SetPaintFolderIcon(FALSE);
	}
    
	GetRCString("PopupSelFolderPix", pszSelFolderPix, pszSelEntryPix, MAX_RC_STRING);
	GetRCString("PopupSeparatorPix", pszSeparatorPix, "", MAX_RC_STRING);
	GetRCString("PopupBottomPix", pszBottomPix, "", MAX_RC_STRING);

	// Base colors
	nTitleColor = GetRCColor("PopupTitleBgColor", GetSysColor(COLOR_ACTIVECAPTION));
	nInActiveTitleColor = GetRCColor("PopupInActiveTitleBgColor", GetSysColor(COLOR_INACTIVECAPTION));
	nEntryColor = GetRCColor("PopupEntryBgColor", GetSysColor(COLOR_BTNFACE));
	nSelEntryColor = GetRCColor("PopupSelEntryBgColor", GetSysColor(COLOR_HIGHLIGHT));
	nFolderColor = GetRCColor("PopupFolderBgColor", nEntryColor);
	nSelFolderColor = GetRCColor("PopupSelFolderBgColor", nSelEntryColor);

	// Gradient colors
	nGradientTitle = GetRCColor("PopupGradientTitle", nTitleColor);
	bGradientTitle = nGradientTitle != nTitleColor;
	nGradientEntry = GetRCColor("PopupGradientEntry", nEntryColor);
	bGradientEntry = nGradientEntry /*!= nEntryColor*/;
	nGradientSelEntry = GetRCColor("PopupGradientSelEntry", nEntryColor);
	bGradientSelEntry = nGradientSelEntry /*!= nSelEntryColor*/;
	nGradientFolder = GetRCColor("PopupGradientFolder", nFolderColor);
	bGradientFolder = nGradientFolder /*!= nFolderColor*/;
	nGradientSelFolder = GetRCColor("PopupGradientSelFolder", nGradientSelEntry);
	bGradientSelFolder = nGradientSelFolder /*!= nFolderColor*/;

	// Text colors
	nTitleTextColor = GetRCColor("PopupTitleColor", GetSysColor(COLOR_WINDOWTEXT));
	nEntryTextColor = GetRCColor("PopupEntryColor", GetSysColor(COLOR_MENUTEXT));
	nSelEntryTextColor = GetRCColor("PopupSelEntryColor", GetSysColor(COLOR_HIGHLIGHTTEXT));
	nFolderTextColor = GetRCColor("PopupFolderColor", nEntryTextColor);
	nSelFolderTextColor = GetRCColor("PopupSelFolderColor", nSelEntryTextColor);

	// 3d decoration colors
	nBevelLightColor = GetRCColor("PopupBevelLightColor", GetSysColor(COLOR_3DHILIGHT));
	nBevelDarkColor = GetRCColor("PopupBevelDarkColor", GetSysColor(COLOR_3DSHADOW));

	// Shadow color
	DWORD nTitleShadowColor = GetRCColor("PopupTitleShadowColor", GetSysColor(COLOR_3DSHADOW));
	DWORD nEntryShadowColor = GetRCColor("PopupEntryShadowColor", GetSysColor(COLOR_3DSHADOW));
	DWORD nSelEntryShadowColor = GetRCColor("PopupSelEntryShadowColor", nEntryShadowColor);
	DWORD nFolderShadowColor = GetRCColor("PopupFolderShadowColor", nEntryShadowColor);
	DWORD nSelFolderShadowColor = GetRCColor("PopupSelFolderShadowColor", nSelEntryShadowColor);

	// Shadow depth
	int nTitleShadow = GetRCInt("PopupTitleShadowDepth", 0);
	int nEntryShadow = GetRCInt("PopupEntryShadowDepth", 0);
	int nSelEntryShadow = GetRCInt("PopupSelEntryShadowDepth", nEntryShadow);
	int nFolderShadow = GetRCInt("PopupFolderShadowDepth", nEntryShadow);
	int nSelFolderShadow = GetRCInt("PopupSelFolderShadowDepth", nSelEntryShadow);

	m_pTitle = MakePainter(pszTitlePix, nTitleColor, nGradientTitle, bGradientTitle, FALSE, nTitleShadow, nTitleShadowColor);
	m_pEntry = MakePainter(pszEntryPix, nEntryColor, nGradientEntry, bGradientEntry, FALSE, nEntryShadow, nEntryShadowColor);
	m_pSelEntry = MakePainter(pszSelEntryPix, nSelEntryColor, nGradientSelEntry, bGradientSelEntry, TRUE, nSelEntryShadow, nSelEntryShadowColor);
	m_pFolder = MakePainter(pszFolderPix, nFolderColor, nGradientFolder, bGradientFolder, FALSE, nFolderShadow, nFolderShadowColor);
	m_pSelFolder = MakePainter(pszSelFolderPix, nSelFolderColor, nGradientSelFolder, bGradientSelFolder, TRUE, nSelFolderShadow, nSelFolderShadowColor);

	if(lstrlen(pszSeparatorPix) > 0) {
		BITMAP bm;
		HBITMAP hbmSeparator = LoadLSImage(pszSeparatorPix, NULL);
		GetObject(hbmSeparator, sizeof(BITMAP), &bm);
		m_pSeparator = new BmpPainter(hbmSeparator, m_bTransparent);
		m_nSeparatorHeight = bm.bmHeight;
	}

	if(lstrlen(pszBottomPix) > 0) {
		BITMAP bm;
		HBITMAP hbmBottom = LoadLSImage(pszBottomPix, NULL);
		GetObject(hbmBottom, sizeof(BITMAP), &bm);
		m_pBottom = new BmpPainter(hbmBottom, m_bTransparent);
		m_nBottomHeight = bm.bmHeight;
	}

	char szArrowImage[MAX_PATH];
	char szSelArrowImage[MAX_PATH];

	GetRCString("PopupArrowImage", szArrowImage, "", MAX_PATH);
	GetRCString("PopupSelArrowImage", szSelArrowImage, szArrowImage, MAX_PATH);

	if(szArrowImage[0])
    {
		m_hbmArrow = LoadLSImage(szArrowImage, NULL);
		m_hbmSelArrow = LoadLSImage(szSelArrowImage, NULL);

		MenuItem::SetArrow(m_hbmArrow);
		MenuItem::SetSelArrow(m_hbmSelArrow);
	}
	else
    {
		m_hbmArrow = NULL;
		m_hbmSelArrow = NULL;
	}

	GetRCString("PopupDefaultIcon", m_pszDefaultIcon, "", MAX_RC_STRING);
    GetRCString("PopupFolderIcon", m_pszFolderIcon, "", MAX_RC_STRING);
	m_bIcons = GetRCBool("PopupIcons", TRUE);
	m_nIconSpacing = GetRCInt("PopupIconSpacing", 4);
	m_nIconSize = GetRCInt("PopupIconSize", 16);

	MenuItem::SetIconSpacing(m_nIconSpacing);
	MenuItem::SetIconSize(m_nIconSize);
	MenuItem::SetIconsOnRight(GetRCBool("PopupIconsOnRight", TRUE));

	m_bAutoSeparator = GetRCBool("PopupAutoSeparator", TRUE);
	m_bShowExtension = GetRCBool("PopupShowExtension", TRUE);

	// Runitem configuration
	RunItem::SetRememberLast(GetRCBool("PopupRunRememberLast", TRUE));

	// Header item configuration
	m_bPinnedAlwaysOnTop = GetRCBool("PinnedPopupNotOnTop", FALSE);
	HeaderItem::SetAlwaysOnTop(m_bPinnedAlwaysOnTop);
	HeaderItem::SetDrawCloseButton(GetRCBool("PopupNoCloseButton", FALSE));

	// Popup heights
	m_nSubmenuHeight = GetRCInt("PopupSubmenuHeight", 20);
	m_nTitleHeight = GetRCInt("PopupTitleHeight", m_nSubmenuHeight);

	// Amp Remote configuration
    m_bAmpNewRemote = GetRCBool("PopupNewAmpRemote", TRUE);

    if (m_bAmpNewRemote)
    {
        GetRCLine("PopupOnAmpPlay", m_sAmpPlayBuffer, MAX_RC_STRING, NULL);
        GetRCLine("PopupOnAmpPause", m_sAmpPauseBuffer, MAX_RC_STRING, NULL);
        GetRCLine("PopupOnAmpPrev", m_sAmpPrevBuffer, MAX_RC_STRING, NULL);
        GetRCLine("PopupOnAmpNext", m_sAmpNextBuffer, MAX_RC_STRING, NULL);
        GetRCLine("PopupOnAmpStop", m_sAmpStopBuffer, MAX_RC_STRING, NULL);
    }

	// Folder item configuration
	int nOverlapX = GetRCInt("PopupOverlapX", 5);
	int nOverlapY = GetRCInt("PopupOverlapY", -m_nSubmenuHeight);
	FolderItem::SetOverlap(nOverlapX, nOverlapY);
	m_bFolderArrow = GetRCBool("PopupNoFolderArrow", FALSE);
	FolderItem::SetDelay(GetRCInt("PopupMenuDelay", 100));
	FolderItem::SetHoverOpen(GetRCBoolDef("PopupFolderHoverOpen", TRUE));

	// Datetime configuration
	m_nDateTimeAlignment = GetRCInt("PopupDateTimeAlign", DT_RIGHT);

	// Popup widths
	int nMinWidth = GetRCInt("PopupMinWidth", 128);
	PopupMenu::SetMinWidth(nMinWidth);

	if (!GetRCBool("PopupAdaptiveWidth", TRUE))
		PopupMenu::SetMaxWidth(nMinWidth);
	else
		PopupMenu::SetMaxWidth(max(nMinWidth, GetRCInt("PopupMaxWidth", 500)));

	// Bitmap size configurations
	BmpPainter::SetLeftBorder( max(GetRCInt("PopupBorderLeft", tmpInteger), 0) );
	BmpPainter::SetRightBorder( max(GetRCInt("PopupBorderRight", tmpInteger), 0) );
	BmpPainter::SetTopBorder( max(GetRCInt("PopupBorderTop", tmpInteger), 0) );
	BmpPainter::SetBottomBorder( max(GetRCInt("PopupBorderBottom", tmpInteger), 0) );

	// Paint mode reading, with new support for PopupPaintMode
	char szPaintMode[32] = {0};
	GetRCString("PopupPaintMode", szPaintMode, "", 32);
	if (!lstrcmpi(szPaintMode, "stretch"))
		BmpPainter::SetBlt(BMPPAINT_STRETCH);
	else if (!lstrcmpi(szPaintMode, "tile"))
		BmpPainter::SetBlt(BMPPAINT_TILE);
	else // if none of the new format, check older
		BmpPainter::SetBlt(GetRCInt("PopupBlt", BMPPAINT_PLAIN));

	TaskItem::SetWindowCaption(GetRCBool("PopupTasksWindowCaption", TRUE));

	LOGFONT lf;
	SystemParametersInfo(SPI_GETICONTITLELOGFONT, 0, &lf, 0);

	// Load the default font face
	int nFontHeight = GetRCInt("PopupFontHeight", lf.lfHeight);
	int nEntryFontHeight = GetRCInt("PopupEntryFontHeight", nFontHeight);
	int nFolderFontHeight = GetRCInt("PopupFolderFontHeight", nEntryFontHeight);
	int nSelEntryFontHeight = GetRCInt("PopupSelEntryFontHeight", nEntryFontHeight);
	int nSelFolderFontHeight = GetRCInt("PopupSelFolderFontHeight", nSelEntryFontHeight);

	GetRCString("PopupFontFace", pszFontFace, lf.lfFaceName, MAX_RC_STRING);
	GetRCString("PopupEntryFontFace", pszEntryFontFace, pszFontFace, MAX_RC_STRING);
	GetRCString("PopupFolderFontFace", pszFolderFontFace, pszEntryFontFace, MAX_RC_STRING);
	GetRCString("PopupSelEntryFontFace", pszSelEntryFontFace, pszEntryFontFace, MAX_RC_STRING);
	GetRCString("PopupSelFolderFontFace", pszSelFolderFontFace, pszSelEntryFontFace, MAX_RC_STRING);

	BOOL bTitleBold = GetRCBoolDef("PopupTitleBold", FALSE);
	BOOL bEntryBold = GetRCBoolDef("PopupEntryBold", FALSE);
	BOOL bFolderBold = GetRCBoolDef("PopupFolderBold", FALSE);
	BOOL bSelEntryBold = GetRCBoolDef("PopupSelEntryBold", FALSE);
	BOOL bSelFolderBold = GetRCBoolDef("PopupSelFolderBold", bSelEntryBold);

	BOOL bTitleUnderline = GetRCBoolDef("PopupTitleUnderline", FALSE);
	BOOL bEntryUnderline = GetRCBoolDef("PopupEntryUnderline", FALSE);
	BOOL bFolderUnderline = GetRCBoolDef("PopupFolderUnderline", FALSE);
	BOOL bSelEntryUnderline = GetRCBoolDef("PopupSelEntryUnderline", FALSE);
	BOOL bSelFolderUnderline = GetRCBoolDef("PopupSelFolderUnderline", bSelEntryUnderline);

	BOOL bTitleItalic = GetRCBoolDef("PopupTitleItalic", FALSE);
	BOOL bEntryItalic = GetRCBoolDef("PopupEntryItalic", FALSE);
	BOOL bFolderItalic = GetRCBoolDef("PopupFolderItalic", FALSE);
	BOOL bSelEntryItalic = GetRCBoolDef("PopupSelEntryItalic", FALSE);
	BOOL bSelFolderItalic = GetRCBoolDef("PopupSelFolderItalic", bSelEntryItalic);

	// Title font
	memset(&lf, 0, sizeof(LOGFONT));
	lf.lfHeight = nFontHeight;
	lf.lfWeight = bTitleBold ? FW_BOLD : FW_NORMAL;
	lf.lfUnderline = bTitleUnderline;
	lf.lfItalic = bTitleItalic;
    lf.lfCharSet = DEFAULT_CHARSET;
	StringCchCopy(lf.lfFaceName, LF_FACESIZE, pszFontFace);
	m_hfnTitle = CreateFontIndirect(&lf);

	// Entry font
	memset(&lf, 0, sizeof(LOGFONT));
	lf.lfHeight = nEntryFontHeight;
	lf.lfWeight = bEntryBold ? FW_BOLD : FW_NORMAL;
	lf.lfUnderline = bEntryUnderline;
	lf.lfItalic = bEntryItalic;
    lf.lfCharSet = DEFAULT_CHARSET;
	StringCchCopy(lf.lfFaceName, LF_FACESIZE, pszEntryFontFace);
	m_hfnEntry = CreateFontIndirect(&lf);

	// Folder font
	memset(&lf, 0, sizeof(LOGFONT));
	lf.lfHeight = nFolderFontHeight;
	lf.lfWeight = bFolderBold ? FW_BOLD : FW_NORMAL;
	lf.lfUnderline = bFolderUnderline;
	lf.lfItalic = bFolderItalic;
    lf.lfCharSet = DEFAULT_CHARSET;
	StringCchCopy(lf.lfFaceName, LF_FACESIZE, pszFolderFontFace);
	m_hfnFolder = CreateFontIndirect(&lf);
	
	// Selected entry font
	memset(&lf, 0, sizeof(LOGFONT));
	lf.lfHeight = nSelEntryFontHeight;
	lf.lfWeight = bSelEntryBold ? FW_BOLD : FW_NORMAL;
	lf.lfUnderline = bSelEntryUnderline;
	lf.lfItalic = bSelEntryItalic;
    lf.lfCharSet = DEFAULT_CHARSET;
	StringCchCopy(lf.lfFaceName, LF_FACESIZE, pszSelEntryFontFace);
	m_hfnSelEntry = CreateFontIndirect(&lf);
	
	// Selected folder font
	memset(&lf, 0, sizeof(LOGFONT));
	lf.lfHeight = nSelFolderFontHeight;
	lf.lfWeight = bSelFolderBold ? FW_BOLD : FW_NORMAL;
	lf.lfUnderline = bSelFolderUnderline;
	lf.lfItalic = bSelFolderItalic;
    lf.lfCharSet = DEFAULT_CHARSET;
	StringCchCopy(lf.lfFaceName, LF_FACESIZE, pszSelFolderFontFace);
	m_hfnSelFolder = CreateFontIndirect(&lf);

	// Set the font
	m_pTitle->SetFont(m_hfnTitle, nTitleTextColor);
	m_pEntry->SetFont(m_hfnEntry, nEntryTextColor);
	m_pSelEntry->SetFont(m_hfnSelEntry, nSelEntryTextColor);
	m_pFolder->SetFont(m_hfnFolder, nFolderTextColor);
	m_pSelFolder->SetFont(m_hfnSelFolder, nSelFolderTextColor);
	
	// Shall bevels be drawn
	MenuItem::DrawBevel(!GetRCBool("PopupNoBevel", TRUE));
	PopupMenu::DrawBevel(!GetRCBool("PopupNoMenuBevel", TRUE));

	// Setup colors to be used for 3d details
	MenuItem::SetLight(nBevelLightColor);
	MenuItem::SetDark(nBevelDarkColor);

	PopupMenu::SetLight(nBevelLightColor);
	PopupMenu::SetDark(nBevelDarkColor);

	MenuItem::SetIndent(GetRCInt("PopupTextOffset", 6));
	MenuItem::SetRightIndent(GetRCInt("PopupRightIndent", 6));

	char szAlignment[32] = {0};
	GetRCString("PopupEntryAlignment", szAlignment, "", 32);

	if (!lstrcmpi(szAlignment, "center"))
		MenuItem::SetAlignment(DT_CENTER);
	else if (!lstrcmpi(szAlignment, "right"))
		MenuItem::SetAlignment(DT_RIGHT);
	else
		MenuItem::SetAlignment(DT_LEFT);

	GetRCString("PopupTitleAlignment", szAlignment, "", 32);

	if (!lstrcmpi(szAlignment, "center"))
		MenuItem::SetTitleAlignment(DT_CENTER);
	else if (!lstrcmpi(szAlignment, "right"))
		MenuItem::SetTitleAlignment(DT_RIGHT);
	else
		MenuItem::SetTitleAlignment(DT_LEFT);

	GetRCString("HotListName", m_szHotListName, RESSTR(IDS_POPUP_HOTLISTNAME), MAX_RC_STRING);
	PopupMenu::SetScrollSpeed(GetRCInt("PopupScrollSpeed", 1));

	// Parse and process *Popup config lines in step.rc
	ParseStepRC();

	// Assign a default root if none explicitly defined
	if(!m_pRoot) {
		if(!m_RootMenus.empty())
			m_pRoot = m_RootMenus[0];
	}

	// Register for LS messages
	if(m_pRoot) {
		int msgs[] = {
		                 LM_POPUP, LM_HIDEPOPUP, LM_GETREVID, 0
		             };

		SendMessage(hParentWnd, LM_REGISTERMESSAGE, (WPARAM) m_pRoot->GetWindow(), (LPARAM) msgs);
	}

	// Create menus for desktops and tasks
	m_pDesktopMenu = new DesktopMenu(RESSTR(IDS_POPUP_DESKTOPS), this);
	m_pDesktopMenu->SetBangName("!PopupDesktops");
	m_RootMenus.push_back(m_pDesktopMenu);

	m_pTaskMenu = new TaskMenu(RESSTR(IDS_POPUP_TASKS), this);
	m_pTaskMenu->SetBangName("!PopupTasks");
	m_pTaskMenu->SetPinnedOnShow(GetRCBool("PopupTasksStartPinned", TRUE) ? true : false);
	m_RootMenus.push_back(m_pTaskMenu);

	SetErrorMode(nErrorMode);
}

void PopupMaker::ParseStepRC()
{
	FILE *pStepRC = LCOpen(NULL);
	
	char szLineBuffer[MAX_LINE_LENGTH];
	char szFirst[MAX_PATH] = { 0 };
	char szSecond[MAX_PATH] = { 0 };
	char szThird[MAX_PATH] = { 0 };
	char szFourth[MAX_PATH] = { 0 };
	char szFifth[MAX_PATH] = { 0 };
	//char extra_text[MAX_PATH] = { 0 };

    char *rgszTokens[] = { szFirst, szSecond, szThird, szFourth, szFifth };

	if(!LCReadNextConfig(pStepRC, "*Popup", szLineBuffer, MAX_LINE_LENGTH))
		return; //Reads step.rc line starting with *Popup into szLineBuffer

	int nTokens = LCTokenize(szLineBuffer, rgszTokens, 5, /*extra_text*/ NULL);
		//Tokenizes contents of szLineBuffer into up to 5 tokens,
		//stores them in rgszTokens with anything beyond 5 in extra_text
	

	if(nTokens >= 2 && (lstrcmpi(szSecond, "!New") == 0 || lstrcmpi(szThird, "!New") == 0))
	{
		// multiple popup mode
		do
		{
			char *pszCaption;
			char *pszBangCommand;
			char *rootCommand; // if after the bangcommand def there is another bang

			// set hotlist name and bang command acc to relevant arguments
			if(nTokens <= 3)
			{
				pszCaption = m_szHotListName;
				pszBangCommand = szThird;
				rootCommand = NULL;
			}
			else if (nTokens >= 4 && lstrcmpi(szSecond, "!New") == 0)
			{
				pszCaption = m_szHotListName;
				pszBangCommand = szThird;
				rootCommand = szFourth;
			}
			else if (nTokens >= 5 && lstrcmpi(szThird, "!New") == 0)
			{
				pszCaption = szSecond;
				pszBangCommand = szFourth;
				rootCommand = szFifth;
			}
			else
			{
				pszCaption = szSecond;
				pszBangCommand = szFourth;
				rootCommand = NULL;
			}
			
			// sets up a new PopupMenu with the spec'd bang command
			PopupMenu *pNewPopup = NULL;

            if (rootCommand)
			{
				if (memicmp(rootCommand, "!PopupFolder:", lstrlen("!PopupFolder:")) == 0)
				{
					char pszFolder[_MAX_PATH];
					LPTSTR pszTemp = rootCommand + lstrlen("!PopupFolder:");
					ExpandEnvironmentStrings(pszTemp, pszFolder, sizeof(pszFolder));
					
					pNewPopup = LoadFolder(pszCaption, pszFolder);
				}
			}
			else
			{
                pNewPopup = new PopupMenu(hInst);
				AddHeaderItem(pNewPopup, pszCaption, TRUE);
				ParseFolder(pStepRC, pNewPopup);
			}

            if (pNewPopup)
            {
                if(pszBangCommand[0])
                    pNewPopup->SetBangName(pszBangCommand);
                
                else
                    m_pRoot = pNewPopup;
                
                pNewPopup->Validate();
                m_RootMenus.push_back(pNewPopup);
            }

            if(!LCReadNextConfig(pStepRC, "*Popup", szLineBuffer, MAX_LINE_LENGTH))
				break;

            szFirst[0] = 0;
			szSecond[0] = 0;
			szThird[0] = 0;
			szFourth[0] = 0;
			szFifth[0] = 0;

			nTokens = LCTokenize(szLineBuffer, rgszTokens, 5, NULL);
		}
		while(nTokens >= 2 && (lstrcmpi(szSecond, "!New") == 0 || lstrcmpi(szThird, "!New") == 0));
	}
	else
	{
		// compatibility mode
		m_pRoot = new PopupMenu(hInst);

		AddHeaderItem(m_pRoot, m_szHotListName, TRUE);
		ParseLine(pStepRC, m_pRoot, szLineBuffer);
		ParseFolder(pStepRC, m_pRoot);
		AddBottomItem(m_pRoot);

		m_pRoot->Validate();
		m_RootMenus.push_back(m_pRoot);
	}

	LCClose(pStepRC);
}

void PopupMaker::ParseFolder(FILE *pStepRC, PopupMenu *pRoot)
{
	while (true)
	{
		char szLineBuffer[MAX_LINE_LENGTH];

		if (!LCReadNextConfig(pStepRC, "*Popup", szLineBuffer, MAX_LINE_LENGTH))
		{
			break;
		}

		if (!ParseLine(pStepRC, pRoot, szLineBuffer))
		{
			break;
		}
	}
}

BOOL PopupMaker::ParseLine(FILE *pStepRC, PopupMenu *pRoot, LPCSTR pszLine)
{
	char szToken1[MAX_LINE_LENGTH], szToken2[MAX_LINE_LENGTH];
    char szToken3[MAX_LINE_LENGTH], szExtra[MAX_LINE_LENGTH];
	
    LPSTR lpszTokens[] = { szToken1, szToken2, szToken3 };

	Painter* pPainter = NULL;
	MenuItem* pMenuItem = NULL;
	LPCSTR pszIcon = NULL;
	char szExplicitIcon[MAX_PATH] = { 0 };
	int nHeight = m_nSubmenuHeight;
	szToken1[0] = szToken2[0] = szToken3[0] = szExtra[0] = '\0';

	int nTokens = LCTokenize(pszLine, lpszTokens, 3, szExtra);

	if (strnicmp(szToken2, ".icon=", 6) == 0)
	{
		StringCchCopy(szExplicitIcon, MAX_PATH, szToken2 + 6);

		char *rgszTokens[1] = { szToken3 };
		char szBuffer[MAX_LINE_LENGTH];

		StringCchCopy(szToken2, MAX_LINE_LENGTH, szToken3);
		StringCchCopy(szBuffer, MAX_LINE_LENGTH, szExtra);
		LCTokenize(szBuffer, rgszTokens, 1, szExtra);

		if (!szBuffer[0])
			nTokens--;
	}

	// special case: separator
	if (nTokens == 2 && lstrcmpi(szToken2, "!Separator") == 0)
	{
		AddSeparator(pRoot);
		return TRUE;
	}

	if (nTokens == 3)
	{
		// *popup "Start Menu" 17 default.bmp "!PopupFolder:C:\WINDOWS\Start Menu"
		if (atoi(szToken3) > 0)
		{
			char pszBmp[MAX_LINE_LENGTH] = {'\0'};
			char pszCmd[MAX_LINE_LENGTH] = {'\0'};
			LPSTR ppszExt[3];
			ppszExt[0] = pszBmp;
			ppszExt[1] = pszCmd;

			nHeight = atoi(szToken3);

			LCTokenize(szExtra, ppszExt, 2, szToken1);
			pPainter = new BmpPainter(LoadLSImage(pszBmp, NULL), m_bTransparent);
			m_Painters.push_back(pPainter);
			StringCchCopy(szToken3, MAX_LINE_LENGTH, pszCmd);
		}

		if (!lstrcmpi(szToken3, "Folder"))
		{
			// Folder
			PopupMenu* pSub = new PopupMenu(hInst);

			AddHeaderItem(pSub, szToken2, FALSE);
			ParseFolder(pStepRC, pSub);
			pSub->Validate();

			pMenuItem = new FolderItem(pSub, szToken2);
            pszIcon = m_pszFolderIcon;
		}
		else if (memicmp(szToken3, "!PopupFolder:", lstrlen("!PopupFolder:")) == 0)
		{
			// PopupFolder
			char pszFolder[_MAX_PATH];
			LPSTR pszTemp = szToken3 + lstrlen("!PopupFolder:");
			ExpandEnvironmentStrings(pszTemp, pszFolder, sizeof(pszFolder));

			//PopupMenu* pSub = LoadFolder(token2, pszFolder, extra_text);
			//pMenuItem = new FolderItem(pSub, token2);
			pMenuItem = new DirectoryFolder(pszFolder, FALSE, this, szToken2);
			pszIcon = m_pszFolderIcon;
		}
		else if (memicmp(szToken3, "!DynamicFolder:", lstrlen("!DynamicFolder:")) == 0)
		{
			// PopupFolder
			char pszFolder[_MAX_PATH];
			LPSTR pszTemp = szToken3 + lstrlen("!DynamicFolder:");
			ExpandEnvironmentStrings(pszTemp, pszFolder, sizeof(pszFolder));

			//PopupMenu* pSub = LoadFolder(pszFolder);
			pMenuItem = new DirectoryFolder(pszFolder, TRUE, this, szToken2);
			pszIcon = m_pszFolderIcon;
		}
		else if (memicmp(szToken3, "!PopupDynamicFolder:", lstrlen("!PopupDynamicFolder:")) == 0)
		{
			// PopupFolder
			char pszFolder[_MAX_PATH];
			LPSTR pszTemp = szToken3 + lstrlen("!PopupDynamicFolder:");
			ExpandEnvironmentStrings(pszTemp, pszFolder, sizeof(pszFolder));

			//PopupMenu* pSub = LoadFolder(pszFolder);
			pMenuItem = new DirectoryFolder(pszFolder, TRUE, this, szToken2);
			pszIcon = m_pszFolderIcon;
		}
		else if(!strcmp(szToken2, "!RemoteAmp"))
		{
			pMenuItem = new RemoteAmpItem(szExtra);
			if(pMenuItem) {
				pPainter = new BmpPainter(LoadLSImage(szToken3, NULL), m_bTransparent);
				m_Painters.push_back(pPainter);
			}
		}
		// if it is a special menu item that wants a bmp background
		else if (szToken2[0] == '!')
		{
			pMenuItem = MakeBangItem(szToken2, NULL);

			// token2 == token3 then there is the thing with lscp bugs..
			if (pMenuItem != NULL && lstrcmpi(szToken2, szToken3) != 0)
			{
				pPainter = new BmpPainter(LoadLSImage(szToken3, NULL), m_bTransparent);
				m_Painters.push_back(pPainter);
			}
		}
		else
		{
			pMenuItem = MakeBangItem(szToken3, szToken2);
			if (pMenuItem == NULL)
			{
				pMenuItem = new CommandItem(szToken3, szExtra, szToken2);
				pszIcon = szToken3[0] == '!' ? NULL : szToken3;
			}
		}

        if (m_bIcons && pMenuItem)
		{
			HICON hIcon = NULL;

			if (szExplicitIcon[0])
			{
				hIcon = GetIconFromPathEx(szExplicitIcon, m_nIconSize, m_nIconSize);
				// hIcon = LoadLSIcon(szExplicitIcon, NULL);
			}
			else if (!pMenuItem->HasIcon())
			{
				if (pszIcon)
					hIcon = GetIconFromPathEx(pszIcon, m_nIconSize, m_nIconSize);
				else if (m_pszDefaultIcon[0])
					hIcon = GetIconFromPathEx(m_pszDefaultIcon, m_nIconSize, m_nIconSize);
				// hIcon = LoadLSIcon(m_pszDefaultIcon, NULL);
			}

			// allow blank (.none) icons
            //if (hIcon)
            pMenuItem->SetIcon(hIcon);
		}

		if (pMenuItem != NULL)
		{
			if (pPainter)
			{
				pMenuItem->SetActivePainter(pPainter);
				pMenuItem->SetPainter(pPainter);
			}
			else
			{
				pMenuItem->SetActivePainter(pMenuItem->IsLeaf() ? m_pSelEntry : m_pSelFolder);
				pMenuItem->SetPainter(pMenuItem->IsLeaf() ? m_pEntry : m_pFolder);
			}

			pMenuItem->SetHeight(nHeight);
			pRoot->AddMenuItem(pMenuItem);
		}
	}
	// *popup !DateTime
	else if (nTokens == 2)
	{
		if (lstrcmpi(szToken2, "~Folder") == 0 || lstrcmpi(szToken2, "~New") == 0)
		{
			AddBottomItem(pRoot);
			return FALSE;
		}
		else
		{
			pMenuItem = MakeBangItem(szToken2, NULL);
		}

		if (pMenuItem != NULL)
		{
			// hack to make the separator the separator size
			/* if(lstrcmpi(token2, "!Separator") == 0)
		{
				if(m_pSeparator)
				{
					pMenuItem->SetActivePainter(m_pSeparator);
					pMenuItem->SetPainter(m_pSeparator);
					pMenuItem->SetHeight(m_nSeparatorHeight);
				}
				else
				{
					pMenuItem->SetActivePainter(m_pSelEntry);
					pMenuItem->SetPainter(m_pEntry);
					pMenuItem->SetHeight(4);
				}
		}
			else
		{
				pMenuItem->SetActivePainter(m_pSelEntry);
				pMenuItem->SetPainter(m_pEntry);
				pMenuItem->SetHeight(m_nSubmenuHeight);
		} */

			pMenuItem->SetActivePainter(pMenuItem->IsLeaf() ? m_pSelEntry : m_pSelFolder);
			pMenuItem->SetPainter(pMenuItem->IsLeaf() ? m_pEntry : m_pFolder);
			pMenuItem->SetHeight(m_nSubmenuHeight);

			if (m_bIcons)
			{
				HICON hIcon = NULL;

				if (szExplicitIcon[0])
                    hIcon = GetIconFromPathEx(szExplicitIcon, m_nIconSize, m_nIconSize);
				else if (m_pszDefaultIcon[0])
					hIcon = GetIconFromPathEx(m_pszDefaultIcon, m_nIconSize, m_nIconSize);

				pMenuItem->SetIcon(hIcon);
			}

			pRoot->AddMenuItem(pMenuItem);
		}
	}

	return TRUE;
}

Painter* PopupMaker::MakePainter(LPSTR pszImage, DWORD dwColor, DWORD dwGradColor, BOOL bGrad, BOOL bSwap, int nShadowDepth, DWORD dwShadowColor)
{
	Painter* pNewPainter = NULL;
	DWORD dwLeft = dwColor;
	DWORD dwRight;

	if (lstrlen(pszImage) > 0)
	{
		pNewPainter = new BmpPainter(LoadLSImage(pszImage, NULL), m_bTransparent);
	}
	else if (dwGradColor != 0x00ff00ff)
	{
		dwRight = dwGradColor;
		if (bSwap)
			swap(dwLeft, dwRight);
		pNewPainter = new GradientPainter(dwLeft, dwRight);
	}
	else if (bGrad)
	{
		dwRight = 0;
		if (bSwap)
			swap(dwLeft, dwRight);
		pNewPainter = new GradientPainter(dwLeft, dwRight);
	}
	else
	{
		pNewPainter = new SolidPainter(dwColor);
	}

	pNewPainter->SetShadow(nShadowDepth, dwShadowColor);
	return pNewPainter;
}

MenuItem* PopupMaker::MakeBangItem(LPSTR pszItem, LPSTR pszTitle)
{
	MenuItem* pMenuItem = NULL;
	/*if (lstrcmpi(pszItem, "!RemoteAmp") == 0)
		pMenuItem = new RemoteAmpItem();
	else */if (lstrcmpi(pszItem, "!DateTime") == 0)
		pMenuItem = new TimeItem(m_nDateTimeAlignment, pszTitle == NULL ? "%c" : pszTitle);
	else if (lstrcmpi(pszItem, "!PopupImage") == 0)
		pMenuItem = new SeparatorItem(FALSE);
	else if (lstrcmpi(pszItem, "!PopupRun") == 0)
		pMenuItem = new RunItem(pszTitle == NULL ? RESSTR(IDS_POPUP_RUN) : pszTitle);
	else if (lstrcmpi(pszItem, "!PopupDesktops") == 0)
		pMenuItem = new FolderItem(new DesktopMenu(pszTitle ? pszTitle : RESSTR(IDS_POPUP_DESKTOPS), this), pszTitle ? pszTitle : RESSTR(IDS_POPUP_DESKTOPS));
	else if (lstrcmpi(pszItem, "!PopupTasks") == 0)
		pMenuItem = new TaskFolder(this, pszTitle == NULL ? RESSTR(IDS_POPUP_TASKS) : pszTitle);
	// else if(lstrcmpi(pszItem, "!Separator") == 0)
	//	pMenuItem = new SeparatorItem(m_pSeparator ? FALSE : TRUE);
	else if (lstrcmpi(pszItem, "!PopupControlPanel") == 0)
		pMenuItem = new ShellFolder(this, pszTitle, CSIDL_CONTROLS);
	else if (lstrcmpi(pszItem, "!PopupMyComputer") == 0)
		pMenuItem = new ShellFolder(this, pszTitle, CSIDL_DRIVES);
	else if (lstrcmpi(pszItem, "!PopupNetwork") == 0)
		pMenuItem = new ShellFolder(this, pszTitle, CSIDL_NETWORK);
	else if (lstrcmpi(pszItem, "!PopupPrinters") == 0)
		pMenuItem = new ShellFolder(this, pszTitle, CSIDL_PRINTERS);
	else if (lstrcmpi(pszItem, "!PopupRecentDocuments") == 0)
		pMenuItem = new ShellFolder(this, pszTitle, CSIDL_RECENT);
	else if (lstrcmpi(pszItem, "!PopupRecycleBin") == 0)
		pMenuItem = new ShellFolder(this, pszTitle, CSIDL_BITBUCKET);
	return pMenuItem;
}

MenuItem *PopupMaker::AddSeparator(PopupMenu *pMenu)
{
	MenuItem *pMenuItem = new SeparatorItem(m_pSeparator ? FALSE : TRUE);

	if (m_pSeparator)
	{
		pMenuItem->SetActivePainter(m_pSeparator);
		pMenuItem->SetPainter(m_pSeparator);
		pMenuItem->SetHeight(m_nSeparatorHeight);
	}
	else
	{
		pMenuItem->SetActivePainter(m_pSelEntry);
		pMenuItem->SetPainter(m_pEntry);
		pMenuItem->SetHeight(4);
	}

	pMenu->AddMenuItem(pMenuItem);
	return pMenuItem;
}

void PopupMaker::AddHeaderItem(PopupMenu* pMenu, LPSTR pszTitle, BOOL bVeto)
{
	if (m_bHeader || bVeto)
	{
		MenuItem* pHead = new HeaderItem(m_bHeaderText ? pszTitle : NULL);
		pHead->SetActivePainter(m_pTitle);
		pHead->SetPainter(m_pTitle);
		pHead->SetHeight(m_nTitleHeight);
        pMenu->AddMenuItem(pHead);
    }
}

void PopupMaker::AddBottomItem(PopupMenu* pMenu)
{
    if (m_pBottom)
    {
        MenuItem* pMenuItem = new SeparatorItem(FALSE);
        pMenuItem->SetPainter(m_pBottom);
        pMenuItem->SetHeight(m_nBottomHeight);
        pMenu->AddMenuItem(pMenuItem);
    }
}

PopupMenu* PopupMaker::LoadFolder(LPSTR pszTitle, LPSTR pszFolder)
{
	//For folder merging
	char seps[] = "|";
	char *token;
    size_t stBufferSize = strlen(pszFolder) + 1;
	LPSTR tempPath = new char[stBufferSize];
	StringCchCopy(tempPath, stBufferSize, pszFolder);

	char szDirectory[MAX_PATH];
	char pszPattern[MAX_PATH];
	long handle;
	_finddata_t found;
	HICON hIcon = NULL;
	MenuItem* pItem = NULL;
    vector<DirectoryFolder*> pDirVector;
	BOOL bFolders = FALSE;
	BOOL bFiles = FALSE;

	// create menu objects
	PopupMenu* pMenu = new PopupMenu(hInst);

	AddHeaderItem(pMenu, pszTitle, FALSE);

	token = strtok(tempPath, seps );
	while (token != NULL)
	{
		// make file search pattern of foldername
		StringCchCopy(pszPattern, MAX_PATH, token);

		// ugly hack but it's better than nothing
		if(!strstr(pszPattern, "*"))
		{
			PathAddBackslash(pszPattern);

			// create search pattern
			StringCchCat(pszPattern, MAX_PATH, "*.*");
		}

		// save directory
		StringCchCopy(szDirectory, MAX_PATH, pszPattern);

		// this doesn't really make sense since we first add the backslash (above) and then remove it only to add it again... oh well.
		PathRemoveFileSpec(szDirectory);
		PathAddBackslash(szDirectory);

/*		// Pattern matching
		if (!pszFilter)
			lstrcat(pszPattern, "*");
		else
		{
			if (strlen(pszFilter) != 0)
				lstrcat(pszPattern, pszFilter);
			else
				lstrcat(pszPattern, "*");
		}*/

		// for all files in the folder, create a CommandItem
		// and add them to the current popupmenu
		handle = _findfirst(pszPattern, &found);

		if (handle != -1)
		{
			do
			{
				if (lstrcmp(found.name, ".") == 0 || lstrcmp(found.name, "..") == 0 || (found.attrib & _A_HIDDEN))
					continue;

				if (found.attrib & _A_SUBDIR)
				{
					bFolders = TRUE;
					char pszSubFolder[_MAX_PATH];
					StringCchCopy(pszSubFolder, _MAX_PATH, token);

					if (pszSubFolder[0] && pszSubFolder[lstrlen(pszSubFolder) - 1] != '\\')
						StringCchCat(pszSubFolder, _MAX_PATH, "\\");

					StringCchCat(pszSubFolder, _MAX_PATH, found.name);

                    vector<DirectoryFolder*>::iterator iter =
                        find_if(pDirVector.begin(), pDirVector.end(),
                                DirectoryFolder::pred_title(found.name));

                    if (iter == pDirVector.end())
                    {
                        pItem = new DirectoryFolder(pszSubFolder, FALSE, this, found.name);
                        pDirVector.push_back(reinterpret_cast<DirectoryFolder*>(pItem));
                    }
                    else
                    {
                        (*iter)->AddPath(pszSubFolder);
                        continue;
                    }

                    if (m_bIcons)
                        pItem->SetIcon(GetIconFromPathEx(m_pszFolderIcon, m_nIconSize, m_nIconSize));
                }
				else
				{
					char pszTitle[_MAX_PATH];
					char pszCmd[_MAX_PATH];

					bFiles = TRUE;
					// Assemble command
					StringCchCopy(pszCmd, _MAX_PATH, szDirectory);

					if (pszCmd[0] && pszCmd[lstrlen(pszCmd) - 1] != '\\')
                        StringCchCat(pszCmd, _MAX_PATH, "\\");

					StringCchCat(pszCmd, _MAX_PATH, found.name);

					// Fix title (_tremove the extension like .lnk)
					StringCchCopy(pszTitle, _MAX_PATH, found.name);
					int nStrLen = lstrlen(pszTitle);
					if (nStrLen > 4 && m_bShowExtension)
					{
						// if we are not to show the extention, just _tremove
						// .lnk or .exe
						LPSTR pszExt = pszTitle + nStrLen - 4;
						if (lstrcmpi(pszExt, ".lnk") == 0 ||
						        lstrcmpi(pszExt, ".pif") == 0 ||
						        lstrcmpi(pszExt, ".bat") == 0 ||
						        lstrcmpi(pszExt, ".exe") == 0 ||
						        lstrcmpi(pszExt, ".com") == 0)
							pszTitle[nStrLen - 4] = '\0';
					}
					else
					{
						PathRemoveExtension(pszTitle);
					}

					pItem = new CommandItem(pszCmd, NULL, pszTitle);

					if (m_bIcons)
					{
						SHFILEINFO shfi;
                        HIMAGELIST hImageList;
/*                        
                        hImageList = (HIMAGELIST) SHGetFileInfo(pszCmd, 0,
                            &shfi, sizeof(SHFILEINFO), SHGFI_ICON | SHGFI_SMALLICON |
                            SHGFI_SYSICONINDEX);
*/
						hImageList = (HIMAGELIST) SHGetFileInfo(pszCmd, 0,
                            &shfi, sizeof(SHFILEINFO), SHGFI_SMALLICON |
                            SHGFI_SYSICONINDEX);
                        // pItem->SetIcon(GetIconFromPath(pszCmd));
                        pItem->SetIcon(hImageList, shfi.iIcon);
                    }
                }
                
                pItem->SetHeight(m_nSubmenuHeight);
                pItem->SetPainter(pItem->IsLeaf() ? m_pEntry : m_pFolder);
                pItem->SetActivePainter(pItem->IsLeaf() ? m_pSelEntry : m_pSelFolder);

                pMenu->AddMenuItem(pItem);

            }
            while (_findnext(handle, &found) == 0);
        }
        _findclose(handle);
        token = strtok(NULL, seps);
    }
    AddBottomItem(pMenu);

	if (m_bAutoSeparator && bFolders && bFiles)
	{
		/* MenuItem *pMenuItem = new SeparatorItem();
		pMenuItem->SetSortPriority(5);

		if(m_pSeparator)
	{
			pMenuItem->SetActivePainter(m_pSeparator);
			pMenuItem->SetPainter(m_pSeparator);
			pMenuItem->SetHeight(m_nSeparatorHeight);
	}
		else
	{
			pMenuItem->SetActivePainter(m_pSelEntry);
			pMenuItem->SetPainter(m_pEntry);
			pMenuItem->SetHeight(4);
	}

		pMenu->AddMenuItem(pMenuItem);
		*/

		MenuItem *pMenuItem = AddSeparator(pMenu);
		pMenuItem->SetSortPriority(5);
	}

	pMenu->Sort();
	pMenu->Validate();

	delete [] tempPath;

	return pMenu;
}
